package org.jvnet.annox.model;

import org.apache.commons.lang.Validate;

public class XClass extends XAnnotatedElement<Class<?>> {

	private final Class<?> targetClass;

	private final XMember<?>[] members;

	private final XConstructor[] constructors;

	private final XField[] fields;

	private final XMethod[] methods;

	public XClass(final Class<?> targetClass, final XAnnotation[] xannotations,
			final XConstructor[] constructors, final XField[] fields,
			final XMethod[] methods) {
		super(targetClass, xannotations);
		Validate.noNullElements(constructors,
				"Constructors must not contain null elements.");
		Validate.noNullElements(fields,
				"Fields must not contain null elements.");
		Validate.noNullElements(methods,
				"Methods must not contain null elements.");
		this.targetClass = targetClass;
		this.constructors = constructors;
		this.fields = fields;
		this.methods = methods;

		this.members = new XMember<?>[constructors.length + fields.length
				+ methods.length];

		System.arraycopy(constructors, 0, members, 0, constructors.length);
		System
				.arraycopy(fields, 0, members, constructors.length,
						fields.length);
		System.arraycopy(methods, 0, members, constructors.length
				+ fields.length, methods.length);

		check();
	}

	public Class<?> getTargetClass() {
		return getAnnotatedElement();
	}

	private void check() {
		for (int index = 0; index < members.length; index++) {
			final XMember<?> member = members[index];
			if (!member.getMember().getDeclaringClass().isAssignableFrom(
					getTargetClass())) {
				throw new IllegalArgumentException("Member [" + member
						+ "] does not belong to the target class ["
						+ targetClass + "].");
			}
		}
	}
}
