package org.jvnet.annox.model;

import java.lang.reflect.Method;

import org.apache.commons.lang.Validate;

public class XMethod extends XMember<Method> {

	private final XParameter[] parameters;

	public XMethod(final Method method, XParameter[] parameters,
			XAnnotation[] xannotations) {
		super(method, xannotations);
		Validate.noNullElements(parameters);
		this.parameters = parameters;
		check();
	}

	public XParameter[] getParameters() {
		return parameters;
	}

	private void check() {

		final Method method = getMethod();
		final XParameter[] parameters = getParameters();
		final Class<?>[] parameterTypes = method.getParameterTypes();
		Validate.isTrue(parameters.length == parameterTypes.length,
				"Wrong number of parameters: [" + parameters.length
						+ "], expected [" + parameterTypes.length + "].");
		for (int index = 0; index < parameters.length; index++) {
			final XParameter parameter = parameters[index];
			final Class<?> parameterType = parameterTypes[index];
			Validate.isTrue(parameterType.equals(parameter.getType()),
					"Wrong parameter type: [" + parameter.getType()
							+ "], expected [" + parameterType + "]");
		}
	}

	public Method getMethod() {
		return getMember();
	}

	public static class XParameter extends XAnnotated {
		private final Class<?> type;

		public XParameter(Class<?> type, XAnnotation[] xannotations) {
			super(xannotations);
			Validate.notNull(type, "Parameter type must not be null.");
			this.type = type;
		}

		public Class<?> getType() {
			return type;
		}

	}

}
